@@ -1,6 +1,5 @@
-use std::{env, process, str::from_utf8, time::Duration};
-
 use futures::{Stream, TryStreamExt};
+use std::{env, process, time::Duration};
 
 use eventsource_client as es;
 
@@ -18,7 +17,7 @@ async fn main() -> Result<(), es::Error> {
     let url = &args[1];
     let auth_header = &args[2];
 
-    let client = es::Client::for_url(url)?
+    let client = es::ClientBuilder::for_url(url)?
         .header("Authorization", auth_header)?
         .reconnect(
             es::ReconnectOptions::reconnect(true)
@@ -30,21 +29,23 @@ async fn main() -> Result<(), es::Error> {
         )
         .build();
 
-    let mut stream = Box::pin(tail_events(client));
+    let mut stream = tail_events(client);
+
     while let Ok(Some(_)) = stream.try_next().await {}
 
     Ok(())
 }
 
-fn tail_events(client: es::Client<es::HttpsConnector>) -> impl Stream<Item = Result<(), ()>> {
+fn tail_events(client: impl es::Client) -> impl Stream<Item = Result<(), ()>> {
     client
         .stream()
-        .map_ok(|event| {
-            println!(
-                "got an event: {}\n{}",
-                event.event_type,
-                from_utf8(event.field("data").unwrap_or_default()).unwrap_or_default()
-            )
+        .map_ok(|event| match event {
+            es::SSE::Event(ev) => {
+                println!("got an event: {}\n{}", ev.event_type, ev.data)
+            }
+            es::SSE::Comment(comment) => {
+                println!("got a comment: \n{}", comment)
+            }
         })
         .map_err(|err| eprintln!("error streaming events: {:?}", err))
 }
