@@ -10,8 +10,7 @@ use super::error::{Error, Result};
 struct EventData {
     pub event_type: String,
     pub data: String,
-    pub comment: Option<String>,
-    pub id: String,
+    pub id: Option<String>,
     pub retry: Option<u64>,
 }
 
@@ -25,7 +24,7 @@ impl EventData {
         self.data.push('\n');
     }
 
-    pub fn with_id(mut self, value: String) -> Self {
+    pub fn with_id(mut self, value: Option<String>) -> Self {
         self.id = value;
         self
     }
@@ -45,10 +44,6 @@ impl TryFrom<EventData> for Option<SSE> {
             return Err(Error::InvalidEvent);
         }
 
-        if event_data.comment.is_some() {
-            return Ok(Some(SSE::Comment(event_data.comment.unwrap())));
-        }
-
         if event_data.data.is_empty() {
             return Ok(None);
         }
@@ -62,11 +57,7 @@ impl TryFrom<EventData> for Option<SSE> {
         let mut data = event_data.data.clone();
         data.truncate(data.len() - 1);
 
-        let id = if event_data.id.is_empty() {
-            None
-        } else {
-            Some(event_data.id.clone())
-        };
+        let id = event_data.id.clone();
 
         let retry = event_data.retry;
 
@@ -151,7 +142,7 @@ pub struct EventParser {
     /// the event data currently being decoded
     event_data: Option<EventData>,
     /// the last-seen event ID; events without an ID will take on this value until it is updated.
-    last_event_id: String,
+    last_event_id: Option<String>,
     sse: VecDeque<SSE>,
 }
 
@@ -162,7 +153,7 @@ impl EventParser {
             incomplete_line: None,
             last_char_was_cr: false,
             event_data: None,
-            last_event_id: String::new(),
+            last_event_id: None,
             sse: VecDeque::with_capacity(3),
         }
     }
@@ -221,7 +212,7 @@ impl EventParser {
                         .get_or_insert_with(|| EventData::new().with_id(id.clone()));
 
                     if key == "comment" {
-                        event_data.comment = Some(value.to_string());
+                        self.sse.push_back(SSE::Comment(value.to_string()));
                     } else if key == "event" {
                         event_data.event_type = value.to_string()
                     } else if key == "data" {
@@ -234,8 +225,13 @@ impl EventParser {
                             continue;
                         }
 
-                        self.last_event_id = value.to_string();
-                        event_data.id = self.last_event_id.clone();
+                        if value.is_empty() {
+                            self.last_event_id = Some("".to_string());
+                        } else {
+                            self.last_event_id = Some(value.to_string());
+                        }
+
+                        event_data.id = self.last_event_id.clone()
                     } else if key == "retry" {
                         match value.parse::<u64>() {
                             Ok(retry) => {
@@ -252,9 +248,7 @@ impl EventParser {
 
                 trace!(
                     "seen empty line, event_data is {:?})",
-                    self.event_data
-                        .as_ref()
-                        .map(|event_data| &event_data.event_type)
+                    event_data.as_ref().map(|event_data| &event_data.event_type)
                 );
 
                 if let Some(event_data) = event_data {
@@ -280,7 +274,6 @@ impl EventParser {
     // incomplete lines from previous chunks.
     fn decode_and_buffer_lines(&mut self, chunk: Bytes) {
         let mut lines = chunk.split_inclusive(|&b| b == b'\n' || b == b'\r');
-
         // The first and last elements in this split are special. The spec requires lines to be
         // terminated. But lines may span chunks, so:
         //  * the last line, if non-empty (i.e. if chunk didn't end with a line terminator),
@@ -288,34 +281,32 @@ impl EventParser {
         //  * the first line should be appended to the incomplete line, if any
 
         if let Some(incomplete_line) = self.incomplete_line.as_mut() {
-            let line = lines
-                .next()
-                // split always returns at least one item
-                .unwrap();
-            trace!(
-                "extending line from previous chunk: {:?}+{:?}",
-                logify(incomplete_line),
-                logify(line)
-            );
+            if let Some(line) = lines.next() {
+                trace!(
+                    "extending line from previous chunk: {:?}+{:?}",
+                    logify(incomplete_line),
+                    logify(line)
+                );
 
-            self.last_char_was_cr = false;
-            if !line.is_empty() {
-                // Checking the last character handles lines where the last character is a
-                // terminator, but also where the entire line is a terminator.
-                match line.last().unwrap() {
-                    b'\r' => {
-                        incomplete_line.extend_from_slice(&line[..line.len() - 1]);
-                        let il = self.incomplete_line.take();
-                        self.complete_lines.push_back(il.unwrap());
-                        self.last_char_was_cr = true;
-                    }
-                    b'\n' => {
-                        incomplete_line.extend_from_slice(&line[..line.len() - 1]);
-                        let il = self.incomplete_line.take();
-                        self.complete_lines.push_back(il.unwrap());
-                    }
-                    _ => incomplete_line.extend_from_slice(line),
-                };
+                self.last_char_was_cr = false;
+                if !line.is_empty() {
+                    // Checking the last character handles lines where the last character is a
+                    // terminator, but also where the entire line is a terminator.
+                    match line.last().unwrap() {
+                        b'\r' => {
+                            incomplete_line.extend_from_slice(&line[..line.len() - 1]);
+                            let il = self.incomplete_line.take();
+                            self.complete_lines.push_back(il.unwrap());
+                            self.last_char_was_cr = true;
+                        }
+                        b'\n' => {
+                            incomplete_line.extend_from_slice(&line[..line.len() - 1]);
+                            let il = self.incomplete_line.take();
+                            self.complete_lines.push_back(il.unwrap());
+                        }
+                        _ => incomplete_line.extend_from_slice(line),
+                    };
+                }
             }
         }
 
@@ -374,6 +365,7 @@ impl EventParser {
 #[cfg(test)]
 mod tests {
     use super::{Error::*, *};
+    use proptest::proptest;
     use test_case::test_case;
 
     fn field<'a>(key: &'a str, value: &'a str) -> Result<Option<(&'a str, &'a str)>> {
@@ -521,9 +513,24 @@ mod tests {
     #[test_case(b":hello\n"; "with LF")]
     #[test_case(b":hello\r"; "with CR")]
     #[test_case(b":hello\r\n"; "with CRLF")]
-    fn test_decode_chunks_comments_are_ignored(chunk: &'static [u8]) {
+    fn test_decode_chunks_comments_are_generated(chunk: &'static [u8]) {
         let mut parser = EventParser::new();
         assert!(parser.process_bytes(Bytes::from(chunk)).is_ok());
+        assert!(parser.get_event().is_some());
+    }
+
+    #[test]
+    fn test_comment_is_separate_from_event() {
+        let mut parser = EventParser::new();
+        let result = parser.process_bytes(Bytes::from(":comment\ndata:hello\n\n"));
+        assert!(result.is_ok());
+
+        let comment = parser.get_event();
+        assert!(matches!(comment, Some(SSE::Comment(_))));
+
+        let event = parser.get_event();
+        assert!(matches!(event, Some(SSE::Event(_))));
+
         assert!(parser.get_event().is_none());
     }
 
@@ -670,4 +677,13 @@ mod tests {
         std::fs::read(format!("test-data/{}", name))
             .unwrap_or_else(|_| panic!("couldn't read {}", name))
     }
+
+    proptest! {
+        #[test]
+        fn test_decode_and_buffer_lines_does_not_crash(next in "(\r\n|\r|\n)*event: [^\n\r:]*(\r\n|\r|\n)", previous in "(\r\n|\r|\n)*event: [^\n\r:]*(\r\n|\r|\n)") {
+            let mut parser = EventParser::new();
+            parser.incomplete_line = Some(previous.as_bytes().to_vec());
+            parser.decode_and_buffer_lines(Bytes::from(next));
+        }
+    }
 }
